//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


//<--- OS DEFINITIONS - DO NOT CHANGE --->

#undef WIN
#undef osname

#ifdef __linux__
#define osname "linux"
#endif

#if defined(_WIN32) || defined(_WIN64) || defined(WIN32) || defined(WIN64)
#define WIN
#define osname "win"
#endif

#ifdef __APPLE__
#define osname "mac"
#endif

#ifdef __sun
#define osname "solaris"
#endif

#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
#define osname "bsd"
#endif

#if !defined(osname)
#warning "<<<< YOUR OPERATING SYSTEM is probably NOT SUPPORTED >>>>"
#endif

//<--- SOME DEFINITIONS - DO NOT CHANGE --->

#ifdef NOSERVICES
#undef SERVICES_ENABLED
#undef SERVICES_ENFORCEPROTECT
#endif

#ifndef false
#define false 0
#endif
#ifndef true
#define true (! false)
#endif
typedef char bool;

#ifndef alloca
#define alloca(x) __builtin_alloca(x)
#endif

#ifndef INET6_ADDRSTRLEN
#define INET6_ADDRSTRLEN 46 //needed for Microsoft Windows
#endif

//<--- EXTERNAL LIBS (SOCKETS, FILESYSTEM & SQLITE) -->

#ifdef SERVICES_ENABLED
//SQlite
#include "sqlite3.h"
#include "sqlite3.c"
#endif
//usleep:
#include <unistd.h>
//atoi-type (windows/unix)
#include <stdlib.h>
#include <stdio.h>
//string functions
#include <string.h>
//file locks
#include <fcntl.h>
//signals
#include <signal.h>
//errno
#include <errno.h>
//waitpid() stuff
#include <sys/types.h>
#ifndef WIN
#include <sys/wait.h>
#endif
//internet connection
#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif
#ifndef SERVICES_ENABLED
#include <stdarg.h> //sqlite seems to include this
#endif

#ifdef WIN
#include <winsock2.h>
#include <windows.h>
#include <ws2tcpip.h>
#include <malloc.h>
#endif

//<--- IMPORTANT GLOBAL VARS --->

unsigned char startupprefix[] = " [****]";
unsigned char startupprefixsub[] = " [    ]";
unsigned char version_copy[50];
char bigversion[] = "0.6";
char version_detail[] = "0.6.0.2";
int willquit = 0;
#define FLOODLINES 8

unsigned char md5result[100];


//<--- IMPORTANT FUNCTION HEADERS --->

int weircdMainLoop();
void weircdUserInit();
void weircdChannelsInit();
void weircdIpsInit();
bool nicknamevalid(char* nick);
#ifdef SERVICES_ENABLED
unsigned char* services_chanowner(const unsigned char* channel);
bool chanregistered(const unsigned char* chan);
bool weircdServicesInit();
unsigned char* services_chanorigname(const unsigned char* channel);
unsigned char* services_chantopic(const unsigned char* channel);
int vrandom(int start, int end);
bool isfakeservicesuser(const unsigned char* usr);
#endif
//void sendserviceswhois(int userid, const unsigned char* svbot);
//void sendwhois(int userid,int targetid);
void motd_send(int uid);
void fatalerror();
int usnd(int user,const unsigned char* line,...);
void moveintochan(int userid,const unsigned char* channame);
#define linkstcp lt //make things easier with this alias

//<--- IMPORTANT PRE-INCLUDED FUNCTIONS --->

//md5 by RSA Data Security
#include "md5.c"
//own string lib
#include "weircd_strings.c"
//own time lib
#include "weircd_time.c"

//<--- SERVER LINKING MODULES --->

//include functions header for weircd functions to be used by modules:
#include "weircd_links_modulewrapper.h"
//include linking modules header:
#include "modules/serverlinking/weircd_modules_linking.h"


//<--- CORE --->

//include WeIRCd core header files
#include "weircd_links_users.h"
#include "weircd_users.h"
#include "weircd_serversockets.h"
#include "weircd_linkdata.h"
#include "weircd_sockets.h"
#include "weircd_links_tcp.h"
#include "weircd_confoptions.h"

//include WeIRCd core files
#include "weircd_signals.c"
#include "weircd_startup.c"
#include "weircd_masks.c"
#include "weircd_channels.c"
#include "weircd_ips.h"
#include "weircd_hostcloaking.c"
#include "weircd_users.c"
#include "weircd_serversockets.c"
#include "weircd_glines.c"
#include "weircd_motd.c"
#include "weircd_readconf.c"
#include "weircd_ips.c"
#include "weircd_usersendbuffer.c"
#include "weircd_mode.c"
#ifdef SERVICES_ENABLED
#include "weircd_services.c"
#endif
#include "weircd_links_users.c"
#include "weircd_links_tcp_send.c"
#include "weircd_links_tcp.c"
#include "weircd_links.c"
#include "weircd_irc.c"
#include "weircd_links_modulewrapper.c"
#include "weircd_sockets.c"
#include "weircd_mainloop.c"

//    ---  end  --- 

